﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Resolvers;

namespace McFragen
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        byte[] localData;

        private void btnGetData_Click(object sender, EventArgs e)
        {
            localData = DownloadSheet(334711, "passwort", "VL", 6);
            tbData.Text = Encoding.UTF7.GetString(localData);
        }

        private byte[] DownloadSheet(int matr, string pass, string type, int id)
        {
            WebClient client = new WebClient();
            client.Encoding = Encoding.ASCII;
			//TODO: Ggf. auf aktuelle Adresse abändern.
            string adress = "http://www2.math.rwth-aachen.de:8068/QuerySheet";
            byte[] result = client.UploadData(adress, Encoding.UTF8.GetBytes("id=" + matr + "&passwd=" + pass + "&format=HTML&resolution=Standard&sheet=" + type + id));// "%DC" == "Ü"
            client.Dispose();
            return result;
        }

        List<Aufgabe> Aufgaben = new List<Aufgabe>();

        private void btnConvert_Click(object sender, EventArgs e)
        {
            tbData.Text = "";
            Aufgaben.AddRange(ConvertData(localData));
        }

        private IEnumerable<Aufgabe> ConvertData(byte[] data)
        {
            XmlReaderSettings settings = new XmlReaderSettings();
            settings.DtdProcessing = DtdProcessing.Parse;
            settings.XmlResolver = new XmlPreloadedResolver(XmlKnownDtds.Xhtml10);
            XmlReader reader = XmlReader.Create(new MemoryStream(data), settings);
            bool solutionIncoming = false;
            bool newTaskIncoming = false;
            bool inTable = false;
            bool ignoreNext = false;
            Aufgabe currentAufgabe = null;
            Teilaufgabe currentTeil = null;
            while (reader.Read())
            {
                if (!inTable && reader.NodeType == XmlNodeType.Element
                    && reader.Name == "table")
                {
                    inTable = true;
                    break;
                }
            }
            while (reader.Read() && inTable) switch (reader.NodeType)
                {
                    case XmlNodeType.Element:
                        if (reader.Name == "img" && !ignoreNext)
                        {
                            string alt = reader.GetAttribute("alt");
                            tbData.Text += alt + "\n";
                            if (alt.Contains("\\begin{array}"))
                            {
                                alt = alt.Replace("&", "& ");
                                alt = alt.Replace("\\\\%", "\\\\\r\n");
                            }
                            if (currentAufgabe.Aufgabentext == null)
                            {
                                currentAufgabe.Aufgabentext = alt;
                            }
                            else
                            {
                                currentTeil = new Teilaufgabe();
                                currentTeil.Aufgabentext = alt;
                                currentAufgabe.Teilaufgaben.Add(currentTeil);
                            }
                        }
                        else if (reader.Name == "span"
                            && reader.GetAttribute("class") == "ergplus")
                        {
                            solutionIncoming = true;
                        }
                        else if (reader.Name == "td")
                        {
                            if (reader.GetAttribute("colspan") == "3")
                            {
                                ignoreNext = true;
                            }
                            if (reader.GetAttribute("align") == "center")
                            {
                                newTaskIncoming = true;
                            }
                        }
                        break;
                    case XmlNodeType.EndElement:
                        if (reader.Name == "span")
                        {
                            solutionIncoming = false;
                        }
                        else if (reader.Name == "td")
                        {
                            newTaskIncoming = false;
                            ignoreNext = false;
                        }
                        else if (reader.Name == "table")
                        {
                            inTable = false;
                        }
                        break;
                    case XmlNodeType.Text:
                        if (solutionIncoming && reader.Value.StartsWith("["))
                        {
                            currentTeil.Lösung = reader.Value.Trim('[', ']');
                            tbData.Text += " -> Lösung:" + currentTeil.Lösung + "\n";
                        }
                        else if (newTaskIncoming)
                        {
                            if (currentAufgabe != null)
                            {
                                yield return currentAufgabe;
                            }
                            currentAufgabe = new Aufgabe();
                            currentAufgabe.Nummer = Convert.ToInt32(reader.Value);
                            tbData.Text += "\n\n---AUFGABE " + currentAufgabe.Nummer + "---\n";
                        }
                        break;
                }
            if (currentAufgabe != null)
            {
                yield return currentAufgabe;
            }
        }

        private void btnAll_Click(object sender, EventArgs e)
        {
			//TODO: Richtige Matrikelnummern und Passwörter einfügen.
            tbData.Text = "Gathering data from " + 123456;
			//TODO: Als Schelifenvariable die Anzahl der Übungsblätter einfügen.
            for (int i = 1; i <= 14; i++)
            {
                Aufgaben.AddRange(ConvertData(DownloadSheet(123456, "passwort", "%DC", i)));
            }
            MehrLösungen(330815, "passwort", "%DC");
            MehrLösungen(334711, "passwort", "%DC");
            CreatePDF();

			//TODO: Gleiches wie oben.
            Aufgaben.Clear();
            tbData.Text = "Gathering data from " + 123456;
            for (int i = 1; i <= 11; i++)
            {
                Aufgaben.AddRange(ConvertData(DownloadSheet(123456, "passwort", "VL", i)));
            }
            MehrLösungen(330815, "passwort", "VL");
            MehrLösungen(334711, "passwort", "VL");
            CreatePDF();
        }

        private void CreatePDF()
        {
            string[] template;
            using (StreamReader reader = new StreamReader("template.tex"))
            {
                template = reader.ReadToEnd().Split(new string[] { "%stop" }, StringSplitOptions.None);
            }
            string outdir = Environment.CurrentDirectory + "\\test" + new Random().Next();
            Directory.CreateDirectory(outdir);
            using (StreamWriter writer = new StreamWriter(outdir + "\\output.tex"))
            {
                writer.Write(template[0]);
                foreach (var Aufgabe in Aufgaben)
                {
                    if (Aufgabe.Teilaufgaben.Count > 0)
                    {
                        writer.Write(template[1].Replace("%num", Aufgabe.Nummer.ToString()).Replace("%aufgabentext", Aufgabe.Aufgabentext));
                        foreach (var Teilaufgabe in Aufgabe.Teilaufgaben)
                        {
                            writer.Write(template[2].Replace("%aufgabentext", Teilaufgabe.Aufgabentext).Replace("%lösung", Teilaufgabe.Lösung));
                        }
                        writer.Write(template[3]);
                    }
                }
                writer.Write(template[4]);
                writer.Close();
            }
			//TODO: Auf dein miktex abändern.
            Process.Start("C:\\Program Files\\MiKTeX 2.9\\miktex\\bin\\x64\\pdflatex.exe", "-output-directory=\"" + outdir + "\" \"" + outdir + "\\output.tex\"").WaitForExit();
        }

        private void MehrLösungen(int matr, string pass, string type)
        {
            tbData.Text = "Gathering data from " + matr;
			//TODO: Wieder die Nummer des letzten Übungsblattes statt 14 einfügen.
            for (int i = 1; i <= 14; i++)
            {
                var neueAufgaben = ConvertData(DownloadSheet(matr, pass, type, i));
                foreach (var neueAufgabe in neueAufgaben)
                {
                    foreach (var bestehendeAufgabe in Aufgaben)
                    {
                        if (bestehendeAufgabe.Nummer == neueAufgabe.Nummer
                            && bestehendeAufgabe.Aufgabentext == neueAufgabe.Aufgabentext)
                        {
                            foreach (var neueTeilaufgabe in neueAufgabe.Teilaufgaben)
                            {
                                if (bestehendeAufgabe.Teilaufgaben.Where(
                                    (Teilaufgabe t) => t.Aufgabentext == neueTeilaufgabe.Aufgabentext).Count() == 0)
                                {
                                    bestehendeAufgabe.Teilaufgaben.Add(neueTeilaufgabe);
                                    break;
                                }
                            }
                            break;
                        }
                    }
                }
            }
        }
    }
}
